<?php

class CBXRBookingEmails {
	/**
	 * The single instance of the class
	 *
	 * @var CBXRBookingEmails
	 */
	private static $_instance = null;

	/**
	 * Array of email notification classes
	 *
	 * @var CBXRBookingEmails[]
	 */
	public $emails = [];

	//public $mail_format;

	/**
	 * Main CBXRBookingEmails Instance.
	 *
	 * Ensures only one instance of CBXRBookingEmails is loaded or can be loaded.
	 *
	 * @return CBXRBookingEmails Main instance
	 * @since 2.1
	 * @static
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}//end method instance

	/**
	 * Cloning is forbidden.
	 *
	 * @since 2.0.0
	 */
	public function __clone() {
		cbxrbooking_doing_it_wrong( __FUNCTION__, __( 'Cloning is forbidden.', 'cbxrbooking' ), '2.0.0' );
	}//end method clone

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since 2.0.0
	 */
	public function __wakeup() {
		cbxrbooking_doing_it_wrong( __FUNCTION__, __( 'Unserializing instances of this class is forbidden.', 'cbxrbooking' ), '2.0.0' );
	}//end method wakeup

	public function __construct() {
		$this->init();

		// Email Header, Footer and content hooks.
		add_action( 'cbxrbooking_email_header', [ $this, 'email_header' ] );
		add_action( 'cbxrbooking_email_footer', [ $this, 'email_footer' ] );

		// Let 3rd parties unhook the above via this hook.
		do_action( 'cbxrbooking_email', $this );
	}//end constructor

	/**
	 * Init email classes.
	 */
	public function init() {
		// Include email classes.
		include_once __DIR__ . '/Emails/CBXRBookingEmail.php';

		$this->emails['new_booking_admin_alert']                    = include __DIR__ . '/Emails/CBXRBookingNewBookingAdminAlertEmail.php';
		$this->emails['new_booking_user_alert']                     = include __DIR__ . '/Emails/CBXRBookingNewBookingUserAlertEmail.php';
		$this->emails['booking_confirm_user_alert']                 = include __DIR__ . '/Emails/CBXRBookingConfirmBookingUserAlertEmail.php';
		$this->emails['booking_cancel_admin_alert']                 = include __DIR__ . '/Emails/CBXRBookingBookingCancelAdminAlert.php';
		$this->emails['booking_cancel_request_user_alert']          = include __DIR__ . '/Emails/CBXRBookingCancelRequestReceivedUserAlert.php';
		$this->emails['booking_cancel_request_approved_user_alert'] = include __DIR__ . '/Emails/CBXRBookingCancelRequestApprovedUserAlert.php';

		$this->emails = apply_filters( 'cbxrbooking_email_classes', $this->emails );
	}//end method init

	/**
	 * Get the email header.
	 *
	 * @param  mixed  $email_heading  Heading for the email.
	 */
	public function email_header( $email_heading ) {
		$template_settings = get_option( 'cbxrbooking_email_tpl' );
		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo cbxrbooking_get_template_html( 'emails/email-header.php', [
			'email_heading'     => $email_heading,
			'template_settings' => $template_settings
		] );
	}//end method email_header

	/**
	 * Get the email footer.
	 */
	public function email_footer() {
		$template_settings = get_option( 'cbxrbooking_email_tpl' );
		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo cbxrbooking_get_template_html( 'emails/email-footer.php', [ 'template_settings' => $template_settings ] );
	}//end method email_footer

	/**
	 * Send the email.
	 *
	 * @param  mixed  $to  Receiver.
	 * @param  mixed  $subject  Email subject.
	 * @param  mixed  $message  Message.
	 * @param  string  $headers  Email headers (default: "Content-Type: text/html\r\n").
	 * @param  string  $attachments  Attachments (default: "").
	 *
	 * @return bool
	 */
	public function send( $to, $subject, $message, $headers = "Content-Type: text/html\r\n", $attachments = '' ) {
		// Send.
		$email = new CBXRBookingEmail();

		return $email->send( $to, $subject, $message, $headers, $attachments );
	}//end method send

	/**
	 * Wraps a message in the cbxrbooking mail template.
	 *
	 * @param  string  $email_heading  Heading text.
	 * @param  string  $message  Email message.
	 * @param  bool  $plain_text  Set true to send as plain text. Default to false.
	 *
	 * @return string
	 */
	public function wrap_message( $email_heading, $message, $plain_text = false ) {
		// Buffer.
		ob_start();

		do_action( 'cbxrbooking_email_header', $email_heading, null );
		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo wpautop( wptexturize( $message ) ); // WPCS: XSS ok.

		do_action( 'cbxrbooking_email_footer', null );

		// Get contents.
		return ob_get_clean();
	}//end method wrap_message


	/**
	 * Get blog name formatted for emails.
	 *
	 * @return string
	 */
	private function get_blogname() {
		return wp_specialchars_decode( get_option( 'blogname' ), ENT_QUOTES );
	}//end method get_blogname

	public function is_user_email() {
		return $this->user_email;
	}//end method is_user_email

	/**
	 * Prepare and send the new booking admin alert.
	 *
	 */
	public function new_booking_admin_alert( $messages, $data, $meta ) {
		$email = $this->emails['new_booking_admin_alert'];

		$email->trigger( $messages, $data, $meta );
	}

	/**
	 * Prepare and send the new booking user alert.
	 *
	 */
	public function new_booking_user_alert( $messages, $data, $meta ) {
		$email = $this->emails['new_booking_user_alert'];

		$status = $email->trigger( $messages, $data, $meta );

		return $status;
	}

	/**
	 * Prepare and send the booking confirmation user alert.
	 *
	 */
	public function booking_confirm_user_alert( $messages, $data, $meta ) {
		$email = $this->emails['booking_confirm_user_alert'];

		$status = $email->trigger( $messages, $data, $meta );

		return $status;
	}

	/**
	 * Prepare and send the booking cancel admin alert.
	 *
	 */
	public function booking_cancel_admin_alert( $messages, $data, $meta, $new_status ) {
		$email = $this->emails['booking_cancel_admin_alert'];

		$status = $email->trigger( $messages, $data, $meta, $new_status );

		return $status;
	}

	/**
	 * Prepare and send the booking cancel request user alert.
	 *
	 */
	public function booking_cancel_request_user_alert( $messages, $data, $meta ) {
		$email = $this->emails['booking_cancel_request_user_alert'];

		$status = $email->trigger( $messages, $data, $meta );

		return $status;
	}

	/**
	 * Prepare and send the booking cancel request approved user alert.
	 *
	 */
	public function booking_cancel_request_approved_user_alert( $messages, $data, $meta ) {
		$email = $this->emails['booking_cancel_request_approved_user_alert'];

		$status = $email->trigger( $messages, $data, $meta );

		return $status;
	}
}//end class CBXRBookingEmails