<?php

namespace CBX\RBooking\Controllers;

use CBX\RBooking\Models\RBranch;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Database\QueryException;
use WP_REST_Request;
use WP_REST_Response;

/**
 * Class BranchController for branch branch listing manage
 */
class BranchController {

	/**
	 * Get branch branch List
	 *
	 * @return WP_REST_Response
	 */
	public function get_branch_list( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			//01. Check if current user is logged in
			if ( ! is_user_logged_in() ) {
				throw new Exception( esc_html__( 'Unauthorized request', 'cbxrbooking' ) );
			}

			global $wpdb;

			$data = $request->get_params();

			$filter             = [];
			$filter['limit']    = isset( $data['limit'] ) ? absint( $data['limit'] ) : 10;
			$filter['page']     = isset( $data['page'] ) ? absint( $data['page'] ) : 1;
			$filter['order_by'] = isset( $data['order_by'] ) ? sanitize_text_field( wp_unslash( $data['order_by'] ) ) : 'id';
			$filter['sort']     = isset( $data['sort'] ) ? sanitize_text_field( wp_unslash( $data['sort'] ) ) : 'desc';
			$filter['search']   = isset( $data['search'] ) ? sanitize_text_field( wp_unslash( $data['search'] ) ) : null;
			$filter['status']   = isset( $data['status'] ) ? sanitize_text_field( wp_unslash( $data['status'] ) ) : null;
			$filter['post_id']  = isset( $data['post_id'] ) ? absint( wp_unslash( $data['post_id'] ) ) : null;

			$sort = strtolower( $filter['sort'] );


			if ( isset( $data['date'] ) && $data['date'] !== '' ) {
				if ( str_contains( $data['date'], ' to ' ) ) {
					$dates = explode( ' to ', $data['date'] );
				} else {
					$dates = $data['date'];
				}
				$filter['date'] = $dates;
			}


			$branches = RBranch::query();

			if ( $filter['search'] ) {
				$filter['search'] = $wpdb->esc_like( $filter['search'] );
				$branches         = $branches->where( 'comment', 'LIKE', '%' . $filter['search'] . '%' );
			}

			if ( isset( $filter['date'] ) && $filter['date'] && is_array( $filter['date'] ) ) {
				$branches = $branches->whereBetween( 'add_date', $filter['date'] );
			}

			if ( isset( $filter['status'] ) && $filter['status'] != null ) {
				$branches = $branches->where( 'status', $filter['status'] );
			}

			if ( isset( $filter['post_id'] ) && $filter['post_id'] != null ) {
				$branches = $branches->where( 'post_id', absint( $filter['post_id'] ) );
			}

			$branches = $branches->orderBy( $filter['order_by'], $sort )->paginate( $filter['limit'], '*', 'page',
				$filter['page'] )->toArray();

			$response->set_data( [
				'success' => true,
				'data'    => $branches,
				'info'    => esc_html__( 'List of branches', 'cbxrbooking' )
			] );

		} catch ( QueryException $e ) {
			// Check if the error is due to a missing table
			if ( str_contains( $e->getMessage(), 'Base table or view not found' ) ) {
				$response->set_data( [
					'info'    => esc_html__( 'Manager table does not exist. Please check the database structure.',
						'cbxrbooking' ),
					'success' => false
				] );
			} else {
				$response->set_data( [
					'info'    => esc_html__( 'Something Went Wrong. Please try again later.', 'cbxrbooking' ),
					'success' => false
				] );
			}
		} catch ( Exception $e ) {
			$response->set_data( [
				'info'    => esc_html__( 'Something Went Wrong. Please try again later.', 'cbxrbooking' ),
				'success' => false
			] );
		}

		return $response;
	} //end method get_branch_list


	/**
	 * Get branch branch data
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 * @throws Exception
	 */
	public function get_branch_data( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );
		global $wpdb;
		//01. Check if current user is logged in
		if ( ! is_user_logged_in() ) {
			throw new Exception( esc_html__( 'Unauthorized request', 'cbxrbooking' ) );
		}


		if ( isset( $request['id'] ) ) {
			$branch = RBranch::where( 'id', absint( $request['id'] ) )->with( 'modUser' )->first();

			if ( $branch ) {
				$branch = $branch->toArray();

				if ( isset( $branch['questions'] ) ) {
					foreach ( $branch['questions'] as $key => $value ) {
						if ( is_serialized( $value ) ) {
							$branch['questions'][ $key ] = maybe_unserialize( $value );
						}
					}
				}

				$response->set_data( $branch );
			} else {
				$response->set_data( [ 'error' => esc_html__( 'Manager Not Found', 'cbxrbooking' ) ] );
			}
		}

		return $response;
	}//end method get_branch_data

	/**
	 * branch Manager delete
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 */
	public function destroy( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		$success_count = $fail_count = 0;

		try {
			if ( ! is_user_logged_in() ) {
				throw new Exception( esc_html__( 'Unauthorized', 'cbxrbooking' ) );
			}

			$data = $request->get_params();
			if ( empty( $data['id'] ) ) {
				throw new Exception( esc_html__( 'Manager id is required.', 'cbxrbooking' ) );
			}


			if ( is_array( $data['id'] ) && count( $data['id'] ) ) {
				foreach ( $data['id'] as $id ) {
					$branch = RBranch::query()->find( absint( $id ) );
					if ( $branch ) {
						if ( $branch->delete() ) {
							$success_count ++;
						} else {
							$fail_count ++;
						}
					}
				}
			} else {
				$branch = RBranch::query()->find( intval( $data['id'] ) );
				if ( $branch ) {
					if ( $branch->delete() ) {
						$success_count ++;
					} else {
						$fail_count ++;
					}

				}
			}

			$success_msg = $fail_msg = '';
			if ( $success_count > 0 ) {
				/* translators: %d: branch successfully deleted count */
				$success_msg = sprintf( esc_html__( '%d branch(s) deleted successfully. ', 'cbxrbooking' ),
					$success_count );

			}

			if ( $fail_count > 0 ) {
				/* translators: %d: branch delete fail count */
				$fail_msg = sprintf( esc_html__( '%d branch(s) can`t be deleted as they may have dependency.',
					'cbxrbooking' ), $fail_count );

			}

			$response->set_data( [
				'success' => true,
				'info'    => $success_msg . $fail_msg
			] );

			return $response;

		} catch ( Exception $e ) {
			$response->set_data( [
				'success' => false,
				'err'     => $e->getMessage(),
				/* translators: 1: Success count 2. Fail count  */
				'info'    => sprintf( esc_html__( 'Incomplete deletion. %1$d successfully and %2$d failed',
					'cbxrbooking' ), $success_count, $fail_count ),
			] );

			return $response;
		}
	} //end method destroy

	/**
	 * save branch Manager
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 */
	public function save( WP_REST_Request $request ) {

		global $wpdb;
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			$data = $request->get_params();

			$validation_errors = [];
			$form_validation   = true;
			$address_input     = [];

			if ( isset( $data['address'] ) && is_array( $data['address'] ) ) {
				foreach ( $data['address'] as $address_key => $address_value ) {
					$address_input[ $address_key ] = sanitize_text_field( $address_value );
				}
			}

			$branch_name        = isset( $data['name'] ) ? sanitize_text_field( $data['name'] ) : '';
			$branch_description = isset( $data['description'] ) ? sanitize_text_field( $data['description'] ) : '';

			$branch_id = isset( $data['id'] ) ? absint( $data['id'] ) : 0;
			$name_len  = mb_strlen( $branch_name );

			if ( $branch_id == 0 ) {
				if ( ! cbxrbooking_check_pro_plugin_active() && RBranch::count() > 0 ) {
					$response->set_data( [
						'info'    => esc_html__( 'Multiple branch creation not allowed.', 'cbxrbooking' ),
						'success' => false,
					] );

					return $response;
				}
			}

			//check branch branch name length is not less than 5 or more than 200 char
			if ( $name_len < 5 || $name_len > 200 ) {
				$response->set_data( [
					'success' => false,
					'info'    => esc_html__( 'The name field character limit must be between 5 to 200', 'cbxrbooking' )
				] );

				return $response;
			}

			//check form passes all validation rules
			if ( $form_validation ) {
				$col_data = [
					'name'        => $branch_name,
					'description' => $branch_description,
					'address'     => maybe_serialize( $address_input ),
				];

				//edit mode
				if ( $branch_id > 0 ) {
					$branch = RBranch::find( $branch_id );
					if ( $branch ) {

						$col_data['mod_by']   = get_current_user_id();
						$col_data['mod_date'] = current_time( 'mysql' );

						$update_status = RBranch::where( 'id', $branch_id )->update( $col_data );

						//matching update function return is false, then update failed.
						if ( ! $update_status ) {
							//update failed
							$response->set_data( [
								'success' => false,
								'info'    => esc_html__( 'Sorry! you don\'t have enough permission to update account.',
									'cbxrbooking' ),
							] );

						} else {
							//update successful							
							$response->set_data( [
								'success' => true,
								'info'    => esc_html__( 'Branch updated.', 'cbxrbooking' ),
							] );
						}
					} else { //if no data exist with id
						$response->set_data( [
							'success' => false,
							'info'    => esc_html__( 'You attempted to edit the branch that doesn\'t exist. ',
								'cbxrbooking' )
						] );
					}
				} else { //if category is new then go here
					$col_data['add_by']   = get_current_user_id();
					$col_data['add_date'] = current_time( 'mysql' );

					//insert new account
					$insert_status = RBranch::query()->create( $col_data );

					if ( $insert_status ) {
						$acc_id = $insert_status->id;


						$response->set_data( [
							'success' => true,
							'info'    => esc_html__( 'Branch created successfully.', 'cbxrbooking' )
						] );

					} else {
						$response->set_data( [
							'success' => false,
							'info'    => esc_html__( 'Something Went Wrong.', 'cbxrbooking' )
						] );
					}
				}

			}
		} catch ( ModelNotFoundException $e ) {
			$response->set_data( [
				'info'    => esc_html__( 'Something Went Wrong. Booking entity not found.', 'cbxrbooking' ),
				'success' => false
			] );
		} catch ( Exception $e ) {
			$response->set_data( [
				'info'    => esc_html__( 'Something Went Wrong. Please try again later.', 'cbxrbooking' ),
				'success' => false
			] );
		}

		return $response;

	}//end method save
}//end class BranchController