<?php

namespace CBX\RBooking\Controllers;

use CBX\RBooking\MigrationManage;
use CBX\RBooking\Helpers\CBXRBookingHelper;
use Exception;
use WP_REST_Request;
use WP_REST_Response;

class DashboardController {

	/**
	 * Get get global overview data
	 *
	 * @param  \WP_REST_Request  $request
	 *
	 * @return \WP_REST_Response
	 * @since 2.0.0
	 */
	public function getGlobalOverviewData( \WP_REST_Request $request ) {
		$response = new \WP_REST_Response();
		$response->set_status( 200 );

		try {
			$data = $request->get_params();

			// Get the current month and year, or fetch them from request params if needed
			$month = isset( $data['month'] ) ? intval( $data['month'] ) : gmdate( 'm' );
			$year  = isset( $data['year'] ) ? intval( $data['year'] ) : gmdate( 'Y' );

			if ( isset( $data['status'] ) ) {
				$status = explode( ',', $data['status'] );
			} else {
				$status = [];
			}

			$daily_booking    = CBXRBookingHelper::getDailyBookingCounts( $year, $month, $status );
			$labels           = array_map( 'strval', array_keys( $daily_booking ) );
			$backgroundColors = array_map( fn() => sprintf( '#%06X', wp_rand( 0, 0xFFFFFF ) ), range( 1, 12 ) );

			// Prepare the chart data format
			$dailyBookingData = [
				'labels'   => $labels,
				'datasets' => [
					[
						'label'           => 'Booking Count',
						'backgroundColor' => $backgroundColors,
						'data'            => $daily_booking
					]
				]
			];


			$response->set_data( [
				'success'          => true,
				'dailyBookingData' => $dailyBookingData,
			] );

			return $response;
		} catch ( \Exception $e ) {
			$response->set_data( [
				'success' => false,
				'info'    => $e->getMessage(),
			] );

			return $response;
		}
	}//end method getGlobalOverviewData

	/**
	 * Get latest listing data
	 *
	 * @param  \WP_REST_Request  $request
	 *
	 * @return \WP_REST_Response
	 * @since 2.0.0
	 */
	public function getLatestListingData( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			$bookings = CBXRBookingHelper::dashboardBookings();

			$response->set_data( [
				'success'  => true,
				'bookings' => $bookings
			] );

			return $response;
		} catch ( Exception $e ) {
			$response->set_data( [
				'success' => false,
				'info'    => $e->getMessage(),
			] );

			return $response;
		}
	}//end method getLatestListingData

	/**
	 * Full plugin option reset
	 */
	public function pluginOptionsReset( WP_REST_Request $request ) {

		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			if ( ! is_user_logged_in() ) {
				throw new Exception( esc_html__( 'Unauthorized', 'cbxrbooking' ) );
			}

			if ( ! current_user_can( 'cbxrbooking_settings_manage' ) ) {
				throw new Exception( esc_html__( 'Sorry, you don\'t have enough permission!', 'cbxrbooking' ) );
			}


			$data = $request->get_params();

			do_action( 'cbxrbooking_plugin_reset_before' );

			//delete options
			$reset_options = isset( $data['reset_options'] ) ? $data['reset_options'] : [];

			foreach ( $reset_options as $key => $option ) {
				if ( $option ) {
					delete_option( $key );
				}
			}

			do_action( 'cbxrbooking_plugin_option_delete' );
			do_action( 'cbxrbooking_plugin_reset_after' );
			do_action( 'cbxrbooking_plugin_reset' );

			$response->set_data( [
				'success' => true,
				'info'    => esc_html__( 'Setting options reset successfully', 'cbxrbooking' )
			] );

			return $response;
		} catch ( Exception $e ) {
			$response->set_data( [
				'success' => false,
				'info'    => $e->getMessage(),
			] );

			return $response;
		}
	} //end plugin_reset

	/**
	 * Full plugin option reset
	 */
	public function runMigration( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			if ( ! is_user_logged_in() ) {
				throw new Exception( esc_html__( 'Unauthorized', 'cbxrbooking' ) );
			}

			if ( ! current_user_can( 'cbxrbooking_settings_manage' ) ) {
				throw new Exception( esc_html__( 'Sorry, you don\'t have enough permission!', 'cbxrbooking' ) );
			}

			MigrationManage::run();

			do_action( 'cbxrbooking_manual_migration_run_after' );

			$response->set_data( [
				'success' => true,
				'info'    => esc_html__( 'Migrated successfully', 'cbxrbooking' )
			] );

			return $response;
		} catch ( Exception $e ) {
			$response->set_data( [
				'success' => false,
				'info'    => $e->getMessage(),
			] );

			return $response;
		}
	} //end runMigration
}//end class DashboardController