<?php

namespace CBX\RBooking\Controllers;

use CBX\RBooking\CBXRBookingAdmin;
use CBX\RBooking\Models\Post;
use CBX\RBooking\Models\RBookingLog;
use CBX\RBooking\Models\PostMetaModel;
use Exception;
use Illuminate\Database\QueryException;
use WP_REST_Request;
use WP_REST_Response;

/**
 * Class FormController for email form listing manage
 */
class FormController {

	/**
	 * Get Email form List
	 *
	 * @return WP_REST_Response
	 */
	public function get_form_list( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			//01. Check if current user is logged in
			if ( ! is_user_logged_in() ) {
				throw new Exception( esc_html__( 'Unauthorized request', 'cbxrbooking' ) );
			}

			global $wpdb;

			$data = $request->get_params();

			$filter             = [];
			$filter['limit']    = isset( $data['limit'] ) ? absint( $data['limit'] ) : 10;
			$filter['page']     = isset( $data['page'] ) ? absint( $data['page'] ) : 1;
			$filter['order_by'] = 'id';
			$filter['sort']     = $sort = isset( $data['sort'] ) ? sanitize_text_field( wp_unslash( $data['sort'] ) ) : 'desc';
			$filter['status']   = isset( $data['status'] ) ? sanitize_text_field( wp_unslash( $data['status'] ) ) : null;
			$filter['branch']   = isset( $data['branch'] ) ? intval( $data['branch'] ) : null;

			$sort = strtolower( $filter['sort'] );

			$forms = Post::bookingForms();

			if ( isset( $filter['status'] ) && $filter['status'] != null ) {
				$forms = $forms->where( 'status', absint( $filter['status'] ) );
			}


			if ( $filter['branch'] ) {
				$form_ids = PostMetaModel::where( 'meta_key', '_cbxrbookingformmeta_branch' )->where( 'meta_value',
					$filter['branch'] )->get()->pluck( 'post_id' )->toArray();
				if ( is_array( $form_ids ) && count( $form_ids ) ) {
					$forms = $forms->whereIn( 'id', $form_ids );
				} else {
					$forms = $forms->where( 'id', 0 );
				}
			}

			$forms = $forms->orderBy( $filter['order_by'], $filter['sort'] )->paginate( $filter['limit'], '*', 'page',
				$filter['page'] )->toArray();

			// $response->set_data($forms);

			$response->set_data( [
				'success' => true,
				'data'    => $forms,
				'info'    => esc_html__( 'List of forms', 'cbxrbooking' )
			] );

		} catch ( QueryException $e ) {
			// Check if the error is due to a missing table
			if ( str_contains( $e->getMessage(), 'Base table or view not found' ) ) {
				$response->set_data( [
					'info'    => esc_html__( 'Form table does not exist. Please check the database structure.',
						'cbxrbooking' ),
					'success' => false
				] );
			} else {
				$response->set_data( [
					'info'    => esc_html__( 'Something Went Wrong. Please try again later.', 'cbxrbooking' ),
					'success' => false
				] );
			}
		} catch ( Exception $e ) {
			$response->set_data( [
				'info'    => esc_html__( 'Something Went Wrong. Please try again later.', 'cbxrbooking' ),
				'success' => false
			] );
		}

		return $response;
	} //end method get_form_list


	/**
	 * Get form fields data
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 * @throws Exception
	 */
	public function get_form_data( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		//01. Check if current user is logged in
		if ( ! is_user_logged_in() ) {
			throw new Exception( esc_html__( 'Unauthorized request', 'cbxrbooking' ) );
		}


		if ( isset( $request['id'] ) ) {
			$form_id = absint( $request['id'] );

			$form       = get_post( $form_id );
			$form_title = get_the_title( $form_id );
			$admin      = new CBXRBookingAdmin();
			$fields     = $admin->cbxrbookingform_meta_settings_fields_by_id( $form_id );

			$meta_values = get_post_meta( $form_id, '_cbxrbookingmeta', true );

			$meta_values = isset( $meta_values['settings'] ) ? $meta_values['settings'] : [];

			$branchId        = get_post_meta( $form_id, '_cbxrbookingformmeta_branch', true );
			$enabled         = get_post_meta( $form_id, '_cbxrbookingformmeta_status', true );
			$submissionCount = absint( get_post_meta( $form_id, '_cbxrbookingmeta_submission_count', true ) );

			if ( $form !== null ) {
				$response->set_data(
					[
						'metaValue'       => $meta_values,
						'form'            => $form,
						'fields'          => $fields,
						'form_title'      => $form_title,
						'branchId'        => $branchId,
						'enabled'         => $enabled,
						'submissionCount' => $submissionCount
					]
				);

			} else {
				$response->set_data( [ 'error' => esc_html__( 'Form Not Found', 'cbxrbooking' ) ] );
			}
		}

		return $response;
	}//end method get_form_data

	/**
	 * Form delete
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 */
	public function destroy( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		$success_count = $fail_count = 0;

		try {
			if ( ! is_user_logged_in() ) {
				throw new Exception( esc_html__( 'Unauthorized', 'cbxrbooking' ) );
			}

			$data = $request->get_params();
			if ( empty( $data['id'] ) ) {
				throw new Exception( esc_html__( 'Form id is required.', 'cbxrbooking' ) );
			}


			if ( is_array( $data['id'] ) && count( $data['id'] ) ) {
				foreach ( $data['id'] as $id ) {
					if ( $this->deleteForm( $id ) ) {
						$success_count ++;
					} else {
						$fail_count ++;
					}

				}
			} else {

				if ( $this->deleteForm( intval( $data['id'] ) ) ) {
					$success_count ++;
				} else {
					$fail_count ++;
				}
			}

			$success_msg = $fail_msg = '';
			if ( $success_count > 0 ) {
				/* translators: %d: form successfully deleted count */
				$success_msg = sprintf( esc_html__( '%d form(s) deleted successfully. ', 'cbxrbooking' ),
					$success_count );

			}

			if ( $fail_count > 0 ) {
				/* translators: %d: form delete fail count */
				$fail_msg = sprintf( esc_html__( '%d form(s) can`t be deleted as they may have dependency.',
					'cbxrbooking' ), $fail_count );

			}

			$response->set_data( [
				'success' => true,
				'info'    => $success_msg . $fail_msg
			] );

			return $response;

		} catch ( Exception $e ) {
			$response->set_data( [
				'success' => false,
				'err'     => $e->getMessage(),
				/* translators: 1: Success count 2. Fail count  */
				'info'    => sprintf( esc_html__( 'Incomplete deletion. %1$d successfully and %2$d failed',
					'cbxrbooking' ), $success_count, $fail_count ),
			] );

			return $response;
		}
	} //end method destroy

	/**
	 * delete Form
	 *
	 * @param  int  $post_id
	 *
	 * @return bool
	 */
	private function deleteForm( $post_id ) {
		//we are deleting a post of type "cbxrbooking"
		//if the form has booking don't allow to delete
		$count = RBookingLog::where( 'form_id', $post_id )->count();

		if ( $count > 0 ) {
			return false;
		} else {
			return wp_delete_post( $post_id, true );
		}
	}

	/**
	 * save Form
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 */
	public function save( WP_REST_Request $request ) {

		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			$reqData = $request->get_params();

			$form_id = isset( $reqData['form']['ID'] ) ? absint( $reqData['form']['ID'] ) : 0;

			if ( $form_id ) {
				$form = get_post( $form_id );

				$post_type = 'cbxrbooking';

				// If this isn't a 'cbxrbooking' post, don't update it.
				if ( $post_type != $form->post_type ) {
					$response->set_data( [
						'info'    => esc_html__( 'Form Not Found', 'cbxrbooking' ),
						'success' => false
					] );

					return $response;
				}

				if ( ! empty( $reqData['cbxrbookingmetabox'] ) ) {
					$metaData = $reqData['cbxrbookingmetabox'];

					$admin  = new CBXRBookingAdmin();
					$fields = $admin->cbxrbookingform_meta_settings_fields_by_id( $form_id );

					foreach ( $fields as $key => $section ) {
						foreach ( $section as $field_key => $field ) {

							if ( isset( $metaData[ $key ][ $field_key ] ) ) {

								$type = isset( $field['type'] ) ? $field['type'] : '';

								if ( in_array( $type, [ 'select', 'text', 'checkbox', 'radio' ] ) ) {
									$metaData[ $key ][ $field_key ] = sanitize_text_field( wp_unslash( $metaData[ $key ][ $field_key ] ) );
								} elseif ( $type == 'number' ) {
									$metaData[ $key ][ $field_key ] = intval( $metaData[ $key ][ $field_key ] );
								} elseif ( $type == 'wysiwyg' ) {
									$metaData[ $key ][ $field_key ] = wp_kses_post( $metaData[ $key ][ $field_key ] );
								} elseif ( $type == 'scheduler' || $type == 'scheduler_exceptions' ) {
									if ( is_array( $metaData[ $key ][ $field_key ] ) ) {
										foreach ( $metaData[ $key ][ $field_key ] as $scheduler_key => $schedule ) {
											foreach ( $schedule as $schedule_field_key => $value ) {
												if ( is_array( $value ) ) {
													foreach ( $value as $sub_key => $sub_value ) {
														$metaData[ $key ][ $field_key ][ $scheduler_key ][ $schedule_field_key ][ $sub_key ] = sanitize_text_field( wp_unslash( $sub_value ) );

														if ( $sub_value < 0 ) {
															unset( $metaData[ $key ][ $field_key ][ $scheduler_key ][ $schedule_field_key ][ $sub_key ] );
														}
													}
												} else {
													$metaData[ $key ][ $field_key ][ $scheduler_key ][ $schedule_field_key ] = sanitize_text_field( wp_unslash( $value ) );
												}

											}
										}
									}
								}
							}

						}
					}

					$postData['settings'] = $metaData;
					$branch_id            = ( isset( $reqData['branchId'] ) && $reqData['branchId'] != null ) ? absint( $reqData['branchId'] ) : 0;
					$enabled              = ( isset( $reqData['enabled'] ) && $reqData['enabled'] != null ) ? absint( $reqData['enabled'] ) : 0;

					Post::where( 'ID', $form_id )->update(
						[
							'post_title'  => $reqData['form']['post_title'],
							'post_status' => $reqData['form']['post_status'],
						]
					);

					update_post_meta( $form_id, '_cbxrbookingmeta', $postData );
					update_post_meta( $form_id, '_cbxrbookingformmeta_branch', $branch_id );
					update_post_meta( $form_id, '_cbxrbookingformmeta_status', $enabled );

					$response->set_data( [
						'info'    => esc_html__( 'Form saved Successfully', 'cbxrbooking' ),
						'success' => true,
						'data'    => $form
					] );

				}
			}

		} catch ( Exception $e ) {
			$response->set_data( [
				'info'    => $e->getMessage(),
				'success' => false
			] );
		}

		return $response;
	}//end method save

	/**
	 * create Form
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 */
	public function create( WP_REST_Request $request ) {

		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			$form_count = wp_count_posts( 'cbxrbooking' );

			//check if multiple Form created if only core installed
			if ( $form_count->publish > 0 ) {
				if ( ! cbxrbooking_check_pro_plugin_active() ) {
					$response->set_data( [
						'info'    => esc_html__( 'Multiple Form Creation not allowed.', 'cbxrbooking' ),
						'success' => false,
					] );

					return $response;
				}
			}

			global $user_ID;
			$new_post = [
				'post_title'  => esc_html__( 'New Booking Form', 'cbxrbooking' ),
				'post_status' => 'draft',
				'post_date'   => gmdate( 'Y-m-d H:i:s' ),
				'post_author' => $user_ID,
				'post_type'   => 'cbxrbooking',
			];

			$post_id = wp_insert_post( $new_post );

			$response->set_data( [
				'info'    => esc_html__( 'Form created Successfully', 'cbxrbooking' ),
				'success' => true,
				'id'      => $post_id
			] );


		} catch ( Exception $e ) {
			$response->set_data( [
				'info'    => $e->getMessage(),
				'success' => false
			] );
		}

		return $response;

	}//end method create

	/**
	 * Form counter reset
	 */
	public function resetCounter( WP_REST_Request $request ) {

		$response = new WP_REST_Response();
		$response->set_status( 200 );

		$reqData = $request->get_params();

		$post_id = ( isset( $reqData['form_id'] ) && $reqData['form_id'] != null ) ? intval( $reqData['form_id'] ) : 0;

		if ( $post_id > 0 ) {
			update_post_meta( $post_id, '_cbxrbookingmeta_submission_count', 0 );
		}

		$response->set_data( [
			'info'    => esc_html__( 'Submission counter reseted successfully', 'cbxrbooking' ),
			'success' => true
		] );

		return $response;
	}//end method resetCounter
}//end class FormController