<?php

namespace CBX\RBooking\Controllers;

use CBX\RBooking\CBXRBookingPublic;
use CBX\RBooking\Helpers\CBXRBookingHelper;
use CBX\RBooking\Models\PostMetaModel;
use CBX\RBooking\Models\RBookingLog;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use WP_REST_Request;
use WP_REST_Response;

/**
 * Class LogController for booking log listing manage
 */
class LogController {

	/**
	 * Get booking log List
	 *
	 * @return WP_REST_Response
	 */
	public function get_log_list( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			//01. Check if current user is logged in
			if ( ! is_user_logged_in() ) {
				throw new Exception( esc_html__( 'Unauthorized request', 'cbxrbooking' ) );
			}

			global $wpdb;

			$data = $request->get_params();

			$filter             = [];
			$filter['limit']    = isset( $data['limit'] ) ? absint( $data['limit'] ) : 10;
			$filter['page']     = isset( $data['page'] ) ? absint( $data['page'] ) : 1;
			$filter['order_by'] = isset( $data['order_by'] ) ? sanitize_text_field( wp_unslash( $data['order_by'] ) ) : 'id';
			$filter['sort']     = isset( $data['sort'] ) ? sanitize_text_field( wp_unslash( $data['sort'] ) ) : 'desc';
			$filter['search']   = isset( $data['search'] ) ? sanitize_text_field( wp_unslash( $data['search'] ) ) : null;
			$filter['status']   = isset( $data['status'] ) ? sanitize_text_field( wp_unslash( $data['status'] ) ) : null;
			$filter['post_id']  = isset( $data['post_id'] ) ? absint( wp_unslash( $data['post_id'] ) ) : null;
			$filter['branch']   = isset( $data['branch'] ) ? intval( $data['branch'] ) : null;
			$filter['form']     = isset( $data['form'] ) ? intval( $data['form'] ) : null;

			$sort = strtolower( $filter['sort'] );


			if ( isset( $data['date'] ) && $data['date'] !== '' ) {
				if ( str_contains( $data['date'], ' to ' ) ) {
					$dates = explode( ' to ', $data['date'] );
				} else {
					$dates = $data['date'];
				}
				$filter['date'] = $dates;
			}


			$logs = RBookingLog::query();

			if ( $filter['search'] ) {
				$filter['search'] = $wpdb->esc_like( $filter['search'] );

				$logs = $logs->where( 'name', 'LIKE', '%' . $filter['search'] . '%' )
				             ->orWhere( 'email', 'LIKE', '%' . $filter['search'] . '%' )
				             ->orWhere( 'secret', 'LIKE', '%' . $filter['search'] . '%' );
			}

			if ( $filter['branch'] ) {
				$form_ids = PostMetaModel::where( 'meta_key', '_cbxrbookingformmeta_branch' )->where( 'meta_value',
					$filter['branch'] )->get()->pluck( 'post_id' )->toArray();
				if ( is_array( $form_ids ) && count( $form_ids ) ) {
					$logs = $logs->whereIn( 'form_id', $form_ids );
				} else {
					$logs = $logs->where( 'form_id', 0 );
				}
			}

			if ( $filter['form'] ) {
				$logs = $logs->where( 'form_id', $filter['form'] );
			}

			if ( isset( $filter['date'] ) && $filter['date'] && is_array( $filter['date'] ) ) {
				$logs = $logs->whereBetween( 'add_date', $filter['date'] );
			}

			if ( isset( $filter['status'] ) && $filter['status'] != null ) {
				$logs = $logs->where( 'status', $filter['status'] );
			}

			if ( isset( $filter['post_id'] ) && $filter['post_id'] != null ) {
				$logs = $logs->where( 'post_id', absint( $filter['post_id'] ) );
			}

			$logs = $logs->orderBy( $filter['order_by'], $sort )->paginate( $filter['limit'], '*', 'page',
				$filter['page'] )->toArray();


			if ( isset( $logs['data'] ) && is_array( $logs['data'] ) ) {
				foreach ( $logs['data'] as $index => $item ) {

					$item['post_title']     = get_the_title( $item['form_id'] );
					$logs['data'][ $index ] = $item;
				}

			}


			$response->set_data( [
				'success' => true,
				'data'    => $logs,
				'info'    => esc_html__( 'List of logs', 'cbxrbooking' )
			] );

		} catch ( QueryException $e ) {
			// Check if the error is due to a missing table
			if ( str_contains( $e->getMessage(), 'Base table or view not found' ) ) {
				$response->set_data( [
					'info'    => esc_html__( 'Log table does not exist. Please check the database structure.',
						'cbxrbooking' ),
					'success' => false
				] );
			} else {
				$response->set_data( [
					'info'    => esc_html__( 'Something Went Wrong. Please try again later.', 'cbxrbooking' ),
					'success' => false
				] );
			}
		} catch ( Exception $e ) {
			$response->set_data( [
				'info'    => esc_html__( 'Something Went Wrong. Please try again later.', 'cbxrbooking' ),
				'success' => false
			] );
		}

		return $response;
	} //end method get_log_list


	/**
	 * Get booking log data
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 * @throws Exception
	 */
	public function get_log_data( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );
		global $wpdb;
		//01. Check if current user is logged in
		if ( ! is_user_logged_in() ) {
			throw new Exception( esc_html__( 'Unauthorized request', 'cbxrbooking' ) );
		}


		if ( isset( $request['id'] ) ) {
			$log = RBookingLog::where( 'id', absint( $request['id'] ) )->with( 'modUser' )->first();

			if ( $log ) {
				$log = $log->toArray();

				if ( isset( $log['questions'] ) ) {
					foreach ( $log['questions'] as $key => $value ) {
						if ( is_serialized( $value ) ) {
							$log['questions'][ $key ] = maybe_unserialize( $value );
						}
					}
				}

				$response->set_data( $log );
			} else {
				$response->set_data( [ 'error' => esc_html__( 'Log Not Found', 'cbxrbooking' ) ] );
			}
		}

		return $response;
	}//end method get_log_data

	/**
	 * booking Log delete
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 */
	public function destroy( WP_REST_Request $request ) {
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		$success_count = $fail_count = 0;

		try {
			if ( ! is_user_logged_in() ) {
				throw new Exception( esc_html__( 'Unauthorized', 'cbxrbooking' ) );
			}

			// //02. Check for user core capability
			// if ( ! current_user_can( 'cbxrbooking_log_delete' ) ) {
			// 	throw new \Exception( esc_html__( 'Sorry, you don\'t have enough permission to do this action', 'cbxrbooking' ) );
			// }

			$data = $request->get_params();
			if ( empty( $data['id'] ) ) {
				throw new Exception( esc_html__( 'Log id is required.', 'cbxrbooking' ) );
			}


			if ( is_array( $data['id'] ) && count( $data['id'] ) ) {
				foreach ( $data['id'] as $id ) {
					$log = RBookingLog::query()->find( absint( $id ) );
					if ( $log ) {
						if ( $log->delete() ) {
							$success_count ++;
						} else {
							$fail_count ++;
						}
					}
				}
			} else {
				$log = RBookingLog::query()->find( intval( $data['id'] ) );
				if ( $log ) {
					if ( $log->delete() ) {
						$success_count ++;
					} else {
						$fail_count ++;
					}

				}
			}

			$success_msg = $fail_msg = '';
			if ( $success_count > 0 ) {
				/* translators: %d: log successfully deleted count */
				$success_msg = sprintf( esc_html__( '%d log(s) deleted successfully. ', 'cbxrbooking' ),
					$success_count );

			}

			if ( $fail_count > 0 ) {
				/* translators: %d: log delete fail count */
				$fail_msg = sprintf( esc_html__( '%d log(s) can`t be deleted as they may have dependency.',
					'cbxrbooking' ), $fail_count );

			}

			$response->set_data( [
				'success' => true,
				'info'    => $success_msg . $fail_msg
			] );

			return $response;

		} catch ( Exception $e ) {
			$response->set_data( [
				'success' => false,
				'err'     => $e->getMessage(),
				/* translators: 1: Success count 2. Fail count  */
				'info'    => sprintf( esc_html__( 'Incomplete deletion. %1$d successfully and %2$d failed',
					'cbxrbooking' ), $success_count, $fail_count ),
			] );

			return $response;
		}
	} //end method destroy

	/**
	 * save booking Log
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 */
	public function save( WP_REST_Request $request ) {

		global $wpdb;
		$response = new WP_REST_Response();
		$response->set_status( 200 );

		$validation_errors = [];

		try {
			$post_data  = $request->get_params();
			$booking_id = isset( $post_data['id'] ) && $post_data['id'] ? absint( $post_data['id'] ) : 0;
			$form_id    = isset( $post_data['form_id'] ) ? intval( $post_data['form_id'] ) : 0;

			if ( $form_id == 0 ) {
				$validation_errors['top_errors']['formid']['formid_empty'] = esc_html__( 'Invalid booking form or booking form doesn\'t exists in backend.',
					'cbxrbooking' );
			}

			// sanitization
			$preferred_date = isset( $post_data['booking_date'] ) ? sanitize_text_field( $post_data['booking_date'] ) : '';
			$preferred_time = isset( $post_data['booking_time'] ) ? sanitize_text_field( $post_data['booking_time'] ) : '';
			$party          = isset( $post_data['party_size'] ) ? intval( $post_data['party_size'] ) : 1;
			$name           = isset( $post_data['name'] ) ? sanitize_text_field( $post_data['name'] ) : '';
			$email          = isset( $post_data['email'] ) ? sanitize_email( $post_data['email'] ) : '';
			$phone          = isset( $post_data['phone'] ) ? sanitize_text_field( $post_data['phone'] ) : '';
			$message        = isset( $post_data['message'] ) ? sanitize_text_field( $post_data['message'] ) : '';

			$current_user = wp_get_current_user();
			$user_id      = $current_user->ID;

			$meta = '';

			// form setting field validation
			if ( $form_id > 0 ) {

				// check post type is cbxrbooking
				$form_id_post_type = get_post_type( $form_id );
				if ( $form_id_post_type === false || $form_id_post_type !== 'cbxrbooking' ) {
					$validation_errors[] = esc_html__( 'Booking form doesn\'t exists in backend.', 'cbxrbooking' );
				}

				// get form setting
				$meta = get_post_meta( $form_id, '_cbxrbookingmeta', true );
				if ( $meta != '' ) {
					if ( $meta['settings']['cbxrbooking_style']['require_name'] === 'name-yes' && empty( $name ) ) {
						$validation_errors[] = esc_html__( 'Please enter your name', 'cbxrbooking' );
					}

					if ( $meta['settings']['cbxrbooking_style']['require_email'] === 'email-yes' && empty( $email ) ) {
						$validation_errors[] = esc_html__( 'Please enter email address', 'cbxrbooking' );
					}

					if ( ! empty( $email ) && ! is_email( $email ) ) {
						$validation_errors[] = esc_html__( 'Please provide valid email address',
							'cbxrbooking' );
					}

					if ( $meta['settings']['cbxrbooking_style']['require_phone'] === 'phone-yes' && empty( $phone ) ) {
						$validation_errors[] = esc_html__( 'Please enter your phone number', 'cbxrbooking' );
					}


					//check if email banned
					if ( isset( $meta['settings']['cbxrbooking_style']['banned_email'] ) ) {
						$banned_emails = $meta['settings']['cbxrbooking_style']['banned_email'];
						if ( is_array( $banned_emails ) && in_array( $email, $banned_emails ) ) {
							$validation_errors[] = esc_html__( 'Sorry! your email address is banned.', 'cbxrbooking' );
						}
					}


					$min_party_size = intval( $meta['settings']['cbxrbooking_style']['min_party_size'] );
					$max_party_size = $meta['settings']['cbxrbooking_style']['max_party_size'] != '' ? intval( $meta['settings']['cbxrbooking_style']['max_party_size'] ) : 100;
					// swap party size if min > max
					if ( $min_party_size > $max_party_size ) {
						$swap           = $max_party_size;
						$max_party_size = $min_party_size;
						$min_party_size = $swap;
					}

					if ( ! ( $party >= $min_party_size && $party <= $max_party_size ) ) {
						$validation_errors[] = esc_html__( 'Party size is not in between minimum and maximum party size',
							'cbxrbooking' );
					}

				}//end if meta exists

				//check if date empty or valid
				if ( empty( $preferred_date ) ) {
					$validation_errors[] = esc_html__( 'Sorry, Date is empty', 'cbxrbooking' );
				} elseif ( ! CBXRBookingHelper::validateDate( $preferred_date, 'Y-m-d' ) ) {
					$validation_errors[] = esc_html__( 'Sorry! Date is invalid.', 'cbxrbooking' );
				}


				//check if time empty or valid
				if ( empty( $preferred_time ) ) {
					$validation_errors[] = esc_html__( 'Sorry! Time is empty.', 'cbxrbooking' );
				} elseif ( ! CBXRBookingHelper::validateDate( $preferred_time, 'H:i' ) ) {
					$validation_errors[] = esc_html__( 'Sorry! Time is invalid.', 'cbxrbooking' );
				}
			}//end if form exists

			$validation_errors = apply_filters( 'cbxrbooking_admin_form_validation_errors', $validation_errors,
				$post_data, $form_id, $booking_id );

			if ( sizeof( $validation_errors ) > 0 ) {
				$response->set_data( [
					'success' => false,
					'info'    => esc_html__( 'Sorry! Validation failed.', 'cbxrbooking' ),
					'errors'  => implode( ', ', $validation_errors )
				] );

				return $response;
			}


			$default_status = isset( $meta['settings']['cbxrbooking_style']['default_state'] ) ? $meta['settings']['cbxrbooking_style']['default_state'] : 'pending';
			$new_status     = isset( $post_data['status'] ) ? sanitize_text_field( $post_data['status'] ) : $default_status;


			$data_safe['form_id']      = $form_id;
			$data_safe['booking_date'] = $preferred_date;
			$data_safe['booking_time'] = $preferred_time;
			$data_safe['name']         = $name;
			$data_safe['email']        = $email;
			$data_safe['party_size']   = $party;
			$data_safe['phone']        = $phone;
			$data_safe['message']      = $message;

			$messages = [];

			//update
			if ( $booking_id > 0 ) {
				//get the old data for the booking log
				$log_data   = RBookingLog::where( 'id', $booking_id )->where( 'form_id', $form_id )->firstOrFail();
				$old_status = $log_data->status;


				$secret = $log_data->secret;

				$meta_data = $log_data->metadata;
				$meta_data = maybe_unserialize( $meta_data );

				$meta_data = apply_filters( 'cbxrbooking_admin_form_meta_data_before_update', $meta_data,
					$post_data, $form_id, $booking_id, $secret );

				$data_safe['user_ip']  = $log_data->user_ip;
				$data_safe['status']   = $new_status;
				$data_safe['metadata'] = maybe_serialize( $meta_data );
				$data_safe['mod_by']   = $user_id;
				$data_safe['mod_date'] = current_time( 'mysql' );

				$data_safe = apply_filters( 'cbxrbooking_admin_form_data_before_update', $data_safe, $form_id,
					$booking_id );

				do_action( 'cbxrbooking_admin_form_before_update', $form_id, $booking_id, $data_safe );

				$update_status = RBookingLog::where( 'id', $booking_id )->where( 'form_id',
					$form_id )->update( $data_safe );

				if ( $update_status ) {
					$data_safe['id']         = $booking_id;
					$data_safe['secret']     = $log_data->secret;
					$data_safe['add_by']     = $log_data->add_by;
					$data_safe['add_date']   = $log_data->add_date;
					$data_safe['activation'] = $log_data->activation;

					do_action( 'cbxrbooking_admin_form_after_update', $form_id, $booking_id, $data_safe );


					$messages[] = [
						'type' => 'success',
						/* translators: %s: secret  */
						'text' => sprintf( __( 'Booking request updated successfully. Booking code: <code>%s</code>',
							'cbxrbooking' ), $secret ),
					];

					//if no status change then we skip sending any email
					if ( $old_status !== $new_status ) {
						do_action( 'cbxrbooking_log_status_to_' . $new_status, $form_id, $post_data, $data_safe,
							$old_status, $new_status, $meta );
						do_action( 'cbxrbooking_log_status_from_' . $old_status . '_to_' . $new_status, $form_id,
							$post_data, $data_safe, $old_status, $new_status, $meta );

						if ( isset( $post_data['email_notify'] ) && intval( $post_data['email_notify'] ) == 1 ) {
							$plugin_public = new CBXRBookingPublic();

							//send email alert to user for booking confirmation email
							if ( $new_status === 'confirmed' ) {
								$messages = $plugin_public->sendFrontendBookingUserEmailAlert( $messages, $data_safe, $meta, $new_status );
							}

							//send email alert to user for booking cancel email
							if ( $new_status == 'canceled' ) {
								$messages = $plugin_public->sendBookingCancelUserEmailAlert( $messages, $data_safe, $meta, $new_status );
							}
						}

					}

					$messages = apply_filters( 'cbxrbooking_backend_validation_success_messages', $messages, $form_id, $booking_id );

					//added new
					$response->set_data( [
						'success'  => true,
						'info'     => esc_html__( 'Booking updated.', 'cbxrbooking' ),
						'messages' => $messages,
					] );

				} else {
					$response->set_data( [
						'success' => false,
						'info'    => esc_html__( 'Sorry! Some problem during updating, please try again.',
							'cbxrbooking' ),
					] );
				}


			} else {
				//insert

				$data_safe['user_ip']    = CBXRBookingHelper::get_ipaddress();
				$data_safe['secret']     = CBXRBookingHelper::generateBookingSecret();
				$data_safe['activation'] = '';
				$data_safe['status']     = $new_status;

				$meta_data             = [];
				$meta_data             = apply_filters( 'cbxrbooking_admin_form_meta_data_before_insert',
					$meta_data, $post_data, $form_id, $booking_id, $data_safe['secret'] );
				$data_safe['metadata'] = maybe_serialize( $meta_data );

				$data_safe['add_by']   = $user_id;
				$data_safe['add_date'] = current_time( 'mysql' );

				$data_safe = apply_filters( 'cbxrbooking_admin_form_data_before_insert', $data_safe, $form_id );

				do_action( 'cbxrbooking_admin_form_before_insert', $form_id, $booking_id );

				$insert_status = RBookingLog::query()->create( $data_safe );

				if ( $insert_status ) {
					$booking_id      = $insert_status->id;
					$data_safe['id'] = $booking_id;

					do_action( 'cbxrbooking_admin_form_after_insert', $form_id, $booking_id );

					//update the form submission count
					$form_submission_count = intval( get_post_meta( $form_id, '_cbxrbookingmeta_submission_count',
						true ) );

					update_post_meta( $form_id, '_cbxrbookingmeta_submission_count',
						intval( $form_submission_count ) + 1 );

					$response->set_data( [
						'success' => true,
						/* translators: %s: secret */
						'info'    => sprintf( wp_kses( __( 'Booking request submitted successfully. Booking code: <code>%s</code>',
							'cbxrbooking' ), [ 'code' => [] ] ), $data_safe['secret'] ),
					] );

					if ( isset( $post_data['email_notify'] ) && intval( $post_data['email_notify'] ) == 1 ) {
						//while we insert we can send email using the public facing email alert function
						$plugin_public = new CBXRBookingPublic();

						//if status is pending or confirmed we will send notification to user, here we don't need to send notification to admin user, again, I don't see any reason to send email for other status
						if ( $data_safe['status'] == 'pending' || $data_safe['status'] == 'confirmed' ) {
							$messages = $plugin_public->sendFrontendBookingUserEmailAlert( $messages, $data_safe,
								$meta, $data_safe['status'] );
						}
					}

					$messages = apply_filters( 'cbxrbooking_backend_validation_success_messages', $messages, $form_id,
						$booking_id );


					$response->set_data( [
						'success'  => true,
						'info'     => esc_html__( 'Booking added.', 'cbxrbooking' ),
						'messages' => $messages,
					] );

				} else {
					//failed to insert
					$response->set_data( [
						'success' => false,
						'info'    => esc_html__( 'Sorry! Some problem during updating, please refresh and try again.',
							'cbxrbooking' ),
					] );
				}
			}//end insert mode

		} catch ( ModelNotFoundException $e ) {
			$response->set_data( [
				'info'    => esc_html__( 'Something Went Wrong. Booking entity not found.', 'cbxrbooking' ),
				'success' => false
			] );
		} catch ( Exception $e ) {
			$response->set_data( [
				'info'    => esc_html__( 'Something Went Wrong. Please try again later.', 'cbxrbooking' ),
				'success' => false
			] );
		}

		return $response;

	}//end method save

	/**
	 * change booking Status
	 *
	 * @param  WP_REST_Request  $request
	 *
	 * @return WP_REST_Response
	 */
	public function changeStatus( WP_REST_Request $request ) {

		$response = new WP_REST_Response();
		$response->set_status( 200 );

		try {
			$post_data = $request->get_params();

			$booking_id = isset( $post_data['id'] ) && $post_data['id'] ? absint( $post_data['id'] ) : 0;
			$status     = isset( $post_data['status'] ) ? sanitize_text_field( $post_data['status'] ) : '';


			if ( ! $status || $booking_id == 0 ) {
				$response->set_data( [
					'success' => false,
					'info'    => esc_html__( 'Sorry! Validation failed.', 'cbxrbooking' ),
				] );

				return $response;
			}


			if ( $booking_id > 0 ) {
				//update status
				$update_status = RBookingLog::where( 'id', $booking_id )->update( [ 'status' => $status ] );

				if ( $update_status ) {

					$response->set_data( [
						'success' => true,
						'info'    => esc_html__( 'Booking status updated successfully.', 'cbxrbooking' ),
					] );

				} else {
					$response->set_data( [
						'success' => false,
						'info'    => esc_html__( 'Sorry! Some problem during updating, please try again.',
							'cbxrbooking' ),
					] );
				}
			}
		} catch ( ModelNotFoundException $e ) {
			$response->set_data( [
				'info'    => esc_html__( 'Something Went Wrong. Booking entity not found.', 'cbxrbooking' ),
				'success' => false
			] );
		} catch ( Exception $e ) {
			$response->set_data( [
				'info'    => esc_html__( 'Something Went Wrong. Please try again later.', 'cbxrbooking' ),
				'success' => false
			] );
		}

		return $response;

	}//end method changeStatus
}//end class LogController