<?php

use CBX\RBooking\Helpers\CBXRBookingHelper;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if ( ! class_exists( 'CBXRBookingNewBookingAdminAlertEmail', false ) ) :

	/**
	 * Class CBXRBookingNewBookingAdminAlertEmail file
	 *
	 * Sending email alert to admin when user submit a new booking
	 */
	class CBXRBookingNewBookingAdminAlertEmail extends CBXRBookingEmail {
		/**
		 * Constructor.
		 */

		protected $meta;

		public function __construct() {
			$this->id          = 'new_booking_admin_alert';
			$this->type        = 2;
			$this->enabled     = 'yes';
			$this->user_email  = false; //alert for admin
			$this->title       = esc_html__( 'New booking admin email alert', 'cbxrbooking' );
			$this->description = esc_html__( 'Sends notification to admin on booking submission.', 'cbxrbooking' );

			$this->template_html = 'emails/new_booking_admin_alert.php';

			$this->placeholders = [
				'{user_name}'       => '',
				'{user_email}'      => '',
				'{booking_date}'    => '',
				'{booking_time}'    => '',
				'{party_size}'      => '',
				'{booking_phone}'   => '',
				'{booking_message}' => '',
				'{booking_ip}'      => '',
				'{booking_code}'    => '',
				'{booking_status}'  => '',
				'{booking_log_url}' => '',
			];

			// Call parent constructor.
			parent::__construct();

			// Other settings.
			$this->recipient = $this->get_option( 'recipient', get_option( 'admin_email' ) );
		}//end method constructor

		/**
		 * Trigger the sending of this email.
		 *
		 */
		public function trigger( $messages, $data, $meta ) {
			$this->object = $data;
			$this->meta   = $meta;

			$settings = $meta['settings'] ?? [];

			$form_email_options = [
				'to'         => $settings['cbxrbooking_email_admin']['to'],
				'cc'         => $settings['cbxrbooking_email_admin']['cc'],
				'bcc'        => $settings['cbxrbooking_email_admin']['bcc'],
				'replyto'    => $settings['cbxrbooking_email_admin']['reply_to'],
				'email_type' => $settings['cbxrbooking_email_admin']['format'],
				'heading'    => $settings['cbxrbooking_email_admin']['heading'],
				'subject'    => $settings['cbxrbooking_email_admin']['subject']
			];

			$this->settings = $form_email_options;


			if ( isset( $settings['cbxrbooking_email_admin']['status'] ) && $settings['cbxrbooking_email_admin']['status'] == 'on' ) {

				$this->placeholders['{user_name}']  = $data['name'];
				$this->placeholders['{user_email}'] = $data['email'];

				$booking_date                         = CBXRBookingHelper::viewDateFormat( $data['booking_date'],
					CBXRBookingHelper::storedFormDateFormatKey( $data['form_id'] ) );
				$this->placeholders['{booking_date}'] = $booking_date;

				$booking_time = $data['booking_time'];

				$twelve_hour_booking_time = CBXRBookingHelper::twelveHourBookingTimeFormat( $data['form_id'], $booking_time );
				if ( $twelve_hour_booking_time != '' ) {
					$booking_time = $twelve_hour_booking_time;
				}

				$this->placeholders['{booking_time}']    = $booking_time;
				$this->placeholders['{party_size}']      = $data['party_size'];
				$this->placeholders['{booking_phone}']   = $data['phone'];
				$this->placeholders['{booking_message}'] = $data['message'];
				$this->placeholders['{booking_ip}']      = $data['user_ip'];
				$this->placeholders['{booking_code}']    = $data['secret'];
				$this->placeholders['{booking_status}']  = CBXRBookingHelper::getBookingStatus( $data['status'] );

				$booking_log_url           = admin_url( 'admin.php?page=cbxrbookinglogs#/log/' . $data['form_id'] . '/' . $data['id'] );
				/* translators: %s: booking link  */
				$booking_log_url_formatted = sprintf( wp_kses( '<p>' . __( 'To check or take action about this booking request %s.', 'cbxrbooking' ) . '</p>',
						[ 'p' => ['a' => ['href' => []]] ]
					),
					'<a href="' . esc_url( $booking_log_url ) . '">' . __( 'click this url', 'cbxrbooking' ) . '</a>'
				);

				$this->placeholders['{booking_log_url}'] = $booking_log_url_formatted;

				$content = $this->get_content();

				$content = apply_filters( 'cbxrbooking_esp_admin_alert', $content, $messages, $data, $meta );

				$this->send( $this->get_recipient(), $this->get_subject(), $content, $this->get_headers(), $this->get_attachments() );
			}
		}//end method trigger

		/**
		 * Get email subject.
		 *
		 * @return string
		 * @since  3.1.0
		 */
		public function get_default_subject() {
			return esc_html__( 'New Booking added', 'cbxrbooking' );
		}//end method get_default_subject

		/**
		 * Get email heading.
		 *
		 * @return string
		 * @since  3.1.0
		 */
		public function get_default_heading() {
			return esc_html__( 'New Booking added', 'cbxrbooking' );
		}//end method get_default_heading

		/**
		 * Default content to show below main email content.
		 *
		 * @return string
		 * @since 3.7.0
		 */
		public function get_default_additional_content() {
			return '';
		}//end method get_default_additional_content

		/**
		 * Get email content.
		 *
		 * @return string
		 */
		public function get_content() {
			//$this->sending = true;

			if ( 'plain' === $this->get_email_type() ) {
				$email_content = wordwrap( preg_replace( $this->plain_search, $this->plain_replace, wp_strip_all_tags( $this->get_content_plain() ) ), 70 );
			} else {
				$email_content = $this->get_content_html();
			}

			return $email_content;
		}//end method get_content

		/**
		 * Get content html.
		 *
		 * @return string
		 */
		public function get_content_html() {
			return cbxrbooking_get_template_html( $this->template_html, [
				'email_heading'      => $this->get_heading(),
				'additional_content' => $this->get_additional_content(),
				'email'              => $this,
			] );
		}//end method get_content_html

		/**
		 * Get content plain.
		 *
		 * @return string
		 */
		public function get_content_plain() {
			$message = $this->get_content_html();

			return \Soundasleep\Html2Text::convert( $message );
		}//end method get_content_plain
	}//end class CBXRBookingNewBookingAdminAlertEmail
endif;

return new CBXRBookingNewBookingAdminAlertEmail();