<?php

use CBX\RBooking\Helpers\CBXRBookingHelper;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if ( ! class_exists( 'CBXRBookingNewBookingUserAlertEmail', false ) ) :

	/**
	 * Class CBXRBookingNewBookingUserAlertEmail file
	 *
	 * Sending email alert to user when user submit a new booking
	 */
	class CBXRBookingNewBookingUserAlertEmail extends CBXRBookingEmail {
		/**
		 * Constructor.
		 */

		protected $meta;

		public function __construct() {
			$this->id          = 'new_booking_user_alert';
			$this->type        = 2;
			$this->enabled     = 'yes';
			$this->user_email  = true; //alert for admin
			$this->title       = esc_html__( 'New booking user email alert', 'cbxrbooking' );
			$this->description = esc_html__( 'Sends notification to user on booking submission.', 'cbxrbooking' );

			$this->template_html = 'emails/new_booking_user_alert.php';

			$this->placeholders = [
				'{user_name}'       => '',
				'{user_email}'      => '',
				'{booking_date}'    => '',
				'{booking_time}'    => '',
				'{party_size}'      => '',
				'{booking_phone}'   => '',
				'{booking_message}' => '',
				'{booking_ip}'      => '',
				'{booking_code}'    => '',
				'{booking_status}'  => '',
				'{booking_log_url}' => '',
			];

			// Call parent constructor.
			parent::__construct();
		}//end method constructor

		public function init_settings2() {
			$email_settings = get_option( 'cbxrbooking_emails', [] );


			$this->settings = isset( $email_settings[ $this->id ] ) ? $email_settings[ $this->id ] : [];


			// If there are no settings defined, use defaults.
			if ( ! is_array( $this->settings ) || ( is_array( $this->settings ) && sizeof( $this->settings ) == 0 ) ) {
				$form_fields    = $this->get_form_fields();
				$this->settings = array_merge( array_fill_keys( array_keys( $form_fields ), '' ), wp_list_pluck( $form_fields, 'default' ) );
			}
		}//end method init_settings

		/**
		 * Trigger the sending of this email.
		 *
		 */
		public function trigger( $messages, $data, $meta ) {

			$this->object = $data;
			$this->meta   = $meta;

			$settings = $meta['settings'] ?? [];

			$form_email_options = [
				'to'         => $data['email'],
				'replyto'    => $settings['cbxrbooking_email_user']['reply_to'],
				'email_type' => $settings['cbxrbooking_email_user']['format'],
				'heading'    => $settings['cbxrbooking_email_user']['new_heading'],
				'subject'    => $settings['cbxrbooking_email_user']['new_subject']
			];

			$this->settings = $form_email_options;

			if ( isset( $settings['cbxrbooking_email_user']['status'] ) && $settings['cbxrbooking_email_user']['status'] == 'on' ) {

				$this->placeholders['{user_name}']  = $data['name'];
				$this->placeholders['{user_email}'] = $data['email'];

				$booking_date                         = CBXRBookingHelper::viewDateFormat( $data['booking_date'],
					CBXRBookingHelper::storedFormDateFormatKey( $data['form_id'] ) );
				$this->placeholders['{booking_date}'] = $booking_date;

				$booking_time = $data['booking_time'];

				$twelve_hour_booking_time = CBXRBookingHelper::twelveHourBookingTimeFormat( $data['form_id'], $booking_time );
				if ( $twelve_hour_booking_time != '' ) {
					$booking_time = $twelve_hour_booking_time;
				}

				$this->recipient                         = $data['email'];
				$this->placeholders['{booking_time}']    = $booking_time;
				$this->placeholders['{party_size}']      = $data['party_size'];
				$this->placeholders['{booking_phone}']   = $data['phone'];
				$this->placeholders['{booking_message}'] = $data['message'];
				$this->placeholders['{booking_code}']    = $data['secret'];
				$this->placeholders['{booking_status}']  = CBXRBookingHelper::getBookingStatus( $data['status'] );

				$activation_link = '';
				if ( $data['activation'] != '' ) {
					$activation_link = add_query_arg(
						[
							'cbxrb_verification' => 1,
							'activation_code'    => $data['activation']
						],
						home_url( '/' )
					);
					/* translators: %s: activation link  */
					$activation_message = __( 'To confirm your booking request, please verify your email address by clicking this url.', 'cbxrbooking' );
					$activation_link_formatted = sprintf( wp_kses( '<p>%s <a href="%s">%s</a></p>', [ 'p' => [], 'a' => [ 'href' => [] ] ] ), esc_html( $activation_message ), esc_url( $activation_link ), __( 'here', 'cbxrbooking' ) );

					$this->placeholders['{activation_link}'] = $activation_link_formatted;
				} else {
					$this->placeholders['{activation_link}'] = $activation_link;
				}

				$cancel_link = '';
				if ( isset( $settings['cbxrbooking_cancel_booking'] ) && $settings['cbxrbooking_cancel_booking']['booking_cancel'] === 'on' ) {
					$cancel_link = add_query_arg( [
						'cbxrb_cancel_request' => 1,
						'secret'               => $data['secret'],
					], home_url( '/' ) );
					/* translators: %s: cancel link  */
					$cancel_message = __( 'To cancel booking please click this url:', 'cbxrbooking' );
					$cancel_link_formatted = sprintf( wp_kses( '<p>%s <a href="%s">%s</a></p>', [ 'p' => [], 'a' => [ 'href' => [] ] ] ), esc_html( $cancel_message ), esc_url( $cancel_link ), __( 'here', 'cbxrbooking' ) );

					$this->placeholders['{cancel_link}'] = $cancel_link_formatted;
				} else {
					$this->placeholders['{cancel_link}'] = $cancel_link;
				}

				$content = $this->get_content();

				$content = apply_filters( 'cbxrbooking_esp_user_alert', $content, $messages, $data, $meta );

				$this->send( $this->get_recipient(), $this->get_subject(), $content, $this->get_headers(), $this->get_attachments() );

				return true;
			}
		}//end method trigger

		/**
		 * Get email subject.
		 *
		 * @return string
		 * @since  3.1.0
		 */
		public function get_default_subject() {
			return esc_html__( 'New Booking Notification', 'cbxrbooking' );
		}//end method get_default_subject

		/**
		 * Get email heading.
		 *
		 * @return string
		 * @since  3.1.0
		 */
		public function get_default_heading() {
			return esc_html__( 'New Booking', 'cbxrbooking' );
		}//end method get_default_heading

		/**
		 * Default content to show below main email content.
		 *
		 * @return string
		 * @since 3.7.0
		 */
		public function get_default_additional_content() {
			return '';
		}//end method get_default_additional_content

		/**
		 * Get email content.
		 *
		 * @return string
		 */
		public function get_content() {

			if ( 'plain' === $this->get_email_type() ) {
				$email_content = wordwrap( preg_replace( $this->plain_search, $this->plain_replace, wp_strip_all_tags( $this->get_content_plain() ) ), 70 );
			} else {
				$email_content = $this->get_content_html();
			}

			return $email_content;
		}//end method get_content

		/**
		 * Get content html.
		 *
		 * @return string
		 */
		public function get_content_html() {
			return cbxrbooking_get_template_html( $this->template_html, [
				'email_heading'      => $this->get_heading(),
				'additional_content' => $this->get_additional_content(),
				'email'              => $this,
			] );
		}//end method get_content_html

		/**
		 * Get content plain.
		 *
		 * @return string
		 */
		public function get_content_plain() {
			$message = $this->get_content_html();

			return \Soundasleep\Html2Text::convert( $message );
		}//end method get_content_plain
	}//end class CBXRBookingNewBookingUserAlertEmail
endif;

return new CBXRBookingNewBookingUserAlertEmail();