<?php

use enshrined\svgSanitize\Sanitizer;

if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'cbxrbooking_deprecated_function' ) ) {
	/**
	 * Wrapper for deprecated functions so we can apply some extra logic.
	 *
	 * @param string $function
	 * @param string $version
	 * @param string $replacement
	 *
	 * @since  2.0.5
	 *
	 */
	function cbxrbooking_deprecated_function( $function, $version, $replacement = null ) {
		if ( defined( 'DOING_AJAX' ) ) {
			do_action( 'deprecated_function_run', $function, $replacement, $version );
			$log_string = "The {$function} function is deprecated since version {$version}."; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			$log_string .= $replacement ? " Replace with {$replacement}." : '';               // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped

			if ( defined( 'WP_DEBUG' ) && WP_DEBUG && defined( 'WP_DEBUG_LOG' ) && WP_DEBUG_LOG ) {
				error_log( $log_string );//phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_error_log
			}
		} else {
			_deprecated_function( $function, $version, $replacement ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		}
	}//end function cbxrbooking_deprecated_function
}

if ( ! function_exists( 'cbxrbooking_is_rest_api_request' ) ) {
	/**
	 * Check if doing rest request
	 *
	 * @return bool
	 */
	function cbxrbooking_is_rest_api_request() {
		if ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
			return true;
		}

		if ( empty( $_SERVER['REQUEST_URI'] ) ) {
			return false;
		}

		$rest_prefix = trailingslashit( rest_get_url_prefix() );

		return ( false !== strpos( sanitize_url(wp_unslash($_SERVER['REQUEST_URI'])), $rest_prefix ) );
	}//end function cbxrbooking_is_rest_api_request
}

if ( ! function_exists( 'cbxrbooking_doing_it_wrong' ) ) {
	/**
	 * Wrapper for _doing_it_wrong().
	 *
	 * @param string $function Function used.
	 * @param string $message Message to log.
	 * @param string $version Version the message was added in.
	 *
	 * @since  1.0.0
	 */
	function cbxrbooking_doing_it_wrong( $function, $message, $version ) {
		// @codingStandardsIgnoreStart
		$message .= ' Backtrace: ' . wp_debug_backtrace_summary();

		if ( wp_doing_ajax() || cbxrbooking_is_rest_api_request() ) {
			do_action( 'doing_it_wrong_run', $function, $message, $version );
			error_log( "{$function} was called incorrectly. {$message}. This message was added in version {$version}." );
		} else {
			_doing_it_wrong( $function, $message, $version );
		}
		// @codingStandardsIgnoreEnd
	}//end function cbxrbooking_doing_it_wrong
}


if ( ! function_exists( 'cbxrbooking_esc_svg' ) ) {
	/**
	 * SVG sanitizer
	 *
	 * @param string $svg_content The content of the SVG file
	 *
	 * @return string|false The SVG content if found, or false on failure.
	 * @since 1.0.0
	 */
	function cbxrbooking_esc_svg( $svg_content = '' ) {
		// Create a new sanitizer instance
		$sanitizer = new Sanitizer();

		return $sanitizer->sanitize( $svg_content );
	}// end method cbxrbooking_esc_svg
}


if ( ! function_exists( 'cbxrbooking_load_svg' ) ) {
	/**
	 * Load an SVG file from a directory.
	 *
	 * @param string $svg_name The name of the SVG file (without the .svg extension).
	 * @param string $directory The directory where the SVG files are stored.
	 *
	 * @return string|false The SVG content if found, or false on failure.
	 * @since 1.0.0
	 */
	function cbxrbooking_load_svg( $svg_name = '', $folder = '' ) {
		//note: code partially generated using chatgpt
		if ( $svg_name == '' ) {
			return '';
		}

		if ( ! function_exists( 'WP_Filesystem' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
		}

		$credentials = request_filesystem_credentials( site_url() . '/wp-admin/', '', false, false, null );
		if ( ! WP_Filesystem( $credentials ) ) {
			return; // Error handling here
		}

		global $wp_filesystem;

		$directory = cbxrbooking_icon_path();

		// Sanitize the file name to prevent directory traversal attacks.
		$svg_name = sanitize_file_name( $svg_name );
		if ( $folder != '' ) {
			$folder = trailingslashit( $folder );
		}

		// Construct the full file path.
		$file_path = $directory . $folder . $svg_name . '.svg';

		$file_path = apply_filters( 'cbxrbooking_svg_file_path', $file_path, $svg_name );

		// Check if the file exists.
		if ( $wp_filesystem->exists( $file_path ) && is_readable( $file_path ) ) {
			// Get the SVG file content.
			return $wp_filesystem->get_contents( $file_path );
		} else {
			// Return false if the file does not exist or is not readable.
			return '';
		}
	}//end method cbxrbooking_load_svg
}

if ( ! function_exists( 'cbxrbooking_icon_path' ) ) {
	/**
	 * Form icon path
	 *
	 * @return mixed|null
	 * @since 1.0.0
	 */
	function cbxrbooking_icon_path() {
		$directory = trailingslashit( CBXRBOOKING_ROOT_PATH ) . 'assets/icons/';

		return apply_filters( 'cbxrbooking_icon_path', $directory );
	}//end method cbxrbooking_icon_path
}


if ( ! function_exists( 'cbxrbooking_all_caps' ) ) {
	/**
	 * All form caps
	 *
	 * @return array
	 * @since 1.0.0
	 */
	function cbxrbooking_all_caps() {
		$all_caps = array_merge( cbxrbooking_log_caps(), cbxrbooking_form_caps(), cbxrbooking_branch_caps() );

		return apply_filters( 'cbxrbooking_all_caps', $all_caps );
	}//end function cbxrbooking_all_caps
}

if ( ! function_exists( 'cbxrbooking_log_caps' ) ) {
	/**
	 * cbxrbooking component capabilities for log manager
	 *
	 * @return array
	 * @since 1.0.0
	 */
	function cbxrbooking_log_caps() {
		//format: plugin_component_verb
		$caps = [
			'cbxrbooking_dashboard_manage',
			'cbxrbooking_settings_manage',
			'cbxrbooking_log_manage',
			'cbxrbooking_log_view',
			'cbxrbooking_log_edit',
			'cbxrbooking_log_delete',
		];

		return apply_filters( 'cbxrbooking_log_caps', $caps );
	}//end function cbxrbooking_log_caps
}

if ( ! function_exists( 'cbxrbooking_form_caps' ) ) {
	/**
	 * cbxrbooking component capabilities for form manager
	 *
	 * @return array
	 * @since 1.0.0
	 */
	function cbxrbooking_form_caps() {
		//format: plugin_component_verb
		$caps = [
			'cbxrbooking_form_manage',
			'cbxrbooking_form_view',
			'cbxrbooking_form_edit',
			'cbxrbooking_form_delete',
		];

		return apply_filters( 'cbxrbooking_form_caps', $caps );
	}//end function cbxrbooking_form_caps
}

if ( ! function_exists( 'cbxrbooking_branch_caps' ) ) {
	/**
	 * cbxrbooking component capabilities for branch manager
	 *
	 * @return array
	 * @since 1.0.0
	 */
	function cbxrbooking_branch_caps() {
		//branchat: plugin_component_verb
		$caps = [
			'cbxrbooking_branch_manage',
			'cbxrbooking_branch_view',
			'cbxrbooking_branch_edit',
			'cbxrbooking_branch_delete',
		];

		return apply_filters( 'cbxrbooking_branch_caps', $caps );
	}//end function cbxrbooking_branch_caps
}

if ( ! function_exists( 'cbxrbooking_check_pro_plugin_active' ) ) {
	function cbxrbooking_check_pro_plugin_active() {
		if ( defined( 'CBXRBOOKINGPRO_PLUGIN_NAME' ) && CBXRBOOKINGPRO_PLUGIN_NAME ) {
			return true;
		}

		return false;
	}
}

if ( ! function_exists( 'cbxrbooking_mailer' ) ) {
	/**
	 * Init the cbxrbooking_mailer
	 *
	 */
	function cbxrbooking_mailer() {
		if ( ! class_exists( 'CBXRBookingEmails' ) ) {
			include_once __DIR__ . '/../CBXRBookingEmails.php';
		}

		return CBXRBookingEmails::instance();
	}//end method cbxrbooking_mailer
}


if ( ! function_exists( 'cbxcbxrbooking_login_url_with_redirect' ) ) {
	function cbxcbxrbooking_login_url_with_redirect() {
		//$login_url          = wp_login_url();
		//$redirect_url       = '';

		if ( is_singular() ) {
			$login_url = wp_login_url( get_permalink() );
			//$redirect_url = get_permalink();
		} else {
			global $wp;
			$login_url = wp_login_url( home_url( add_query_arg( [], $wp->request ) ) );
			//$redirect_url = home_url( add_query_arg( [], $wp->request ) );
		}

		return $login_url;
	}//end function cbxcbxrbooking_login_url_with_redirect
}

if ( ! function_exists( 'cbxrbooking_check_version_and_deactivate_plugin' ) ) {
	/**
	 * Check any plugin active, check version, if less than x then deactivate
	 *
	 * @param string $plugin_slug plugin slug
	 * @param string $required_version required plugin version
	 * @param string $transient transient name
	 *
	 * @return bool|void
	 * @since 2.0.0
	 */
	function cbxrbooking_check_version_and_deactivate_plugin( $plugin_slug = '', $required_version = '', $transient = '' ) {
		if ( $plugin_slug == '' ) {
			return;
		}

		if ( $required_version == '' ) {
			return;
		}

		if ( ! function_exists( 'is_plugin_active' ) ) {
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		// Check if the plugin is active
		if ( is_plugin_active( $plugin_slug ) ) {
			// Get the plugin data
			$plugin_data    = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin_slug );
			$plugin_version = isset( $plugin_data['Version'] ) ? $plugin_data['Version'] : '';
			if ( $plugin_version == '' || is_null( $plugin_version ) ) {
				return;
			}

			// Compare the plugin version with the required version
			if ( version_compare( $plugin_version, $required_version, '<' ) ) {
				// Deactivate the plugin
				deactivate_plugins( $plugin_slug );
				if ( $transient != '' ) {
					set_transient( $transient, 1 );
				}
			}
		}

		//return false;
	}//end method cbxrbooking_check_version_and_deactivate_plugin
}

if ( ! function_exists( 'cbxrbooking_version_and_deactivate_plugin' ) ) {
	/**
	 * Check if any plugin activated and then deactivate
	 *
	 * @param string $plugin_slug plugin slug
	 * @param string $transient transient name
	 *
	 * @return bool|void
	 * @since 2.0.0
	 */
	function cbxrbooking_version_and_deactivate_plugin( $plugin_slug = '', $transient = '' ) {
		if ( $plugin_slug == '' ) {
			return;
		}


		if ( ! function_exists( 'is_plugin_active' ) ) {
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		// Check if the plugin is active
		if ( is_plugin_active( $plugin_slug ) ) {
			// Get the plugin data
			$plugin_data    = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin_slug );
			$plugin_version = isset( $plugin_data['Version'] ) ? $plugin_data['Version'] : '';
			if ( $plugin_version == '' || is_null( $plugin_version ) ) {
				return;
			}


			// Deactivate the plugin
			deactivate_plugins( $plugin_slug );
			if ( $transient != '' ) {
				set_transient( $transient, 1 );
			}
		}

		//return false;
	}//end method cbxrbooking_version_and_deactivate_plugin
}